@@ -4,30 +4,36 @@ use strict;
 use vars qw($VERSION $DEBUG);
 use base qw(Class::Accessor);
 Finance::Bank::DE::NetBank->mk_accessors(
-    qw(BASE_URL BLZ CUSTOMER_ID PASSWORD AGENT_TYPE AGENT ACCOUNT Debug));
+    qw(BASE_URL BLZ CUSTOMER_ID PASSWORD AGENT_TYPE AGENT ACCOUNT));
 
 use WWW::Mechanize;
+use HTML::TreeBuilder;
 use Text::CSV_XS;
 use Data::Dumper;
 
 $| = 1;
 
-$VERSION = "1.05_01";
+$VERSION = "1.01";
+$DEBUG   = 0;
 
 sub Version {
     return $VERSION;
 }
 
+sub Debug {
+    $_[0] ? $DEBUG = $_[0] : return $DEBUG;
+}
+
 sub new {
     my $proto  = shift;
     my %values = (
-        AGENT_TYPE => "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) ",
         BASE_URL =>
-          'https://www.netbank-money.de/netbank-barrierefrei-banking/view/',
-        BLZ         => '20090500', # NetBank BLZ
-        CUSTOMER_ID => '',
-        PASSWORD    => '',
-        ACCOUNT     => '', 
+          "https://www.netbank-money.de/netbank-barrierefrei-banking/view/",
+        BLZ         => "20090500",    # NetBank BLZ
+        CUSTOMER_ID => "demo",        # Demo Login
+        PASSWORD    => "",            # Demo does not require a password
+        ACCOUNT     => "1234567",     # Demo Account Number (Kontonummer)
+        AGENT_TYPE => "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) ",
         @_
     );
 
@@ -46,8 +52,10 @@ sub new {
 
 sub connect {
     my $self  = shift;
-    print STDERR "Method connect() is deprecated. Use only login() instead!\n";
-    return $self->login(@_);
+    my $url   = $self->BASE_URL() . "index.jsp?blz=" . $self->BLZ();
+    my $agent = WWW::Mechanize->new( agent => $self->AGENT_TYPE(), );
+    $agent->get($url);
+    $self->AGENT($agent);
 }
 
 sub login {
@@ -57,36 +65,20 @@ sub login {
         PASSWORD    => $self->PASSWORD(),
         @_
     );
-   
-    my $url   = $self->BASE_URL() . "index.jsp?blz=" . $self->BLZ();
-    my $agent = WWW::Mechanize->new( agent => $self->AGENT_TYPE(), );
-    $agent->get($url);
-    $self->AGENT($agent);
 
+    my $agent = $self->AGENT();
     $agent->field( "kundennummer", $values{'CUSTOMER_ID'} );
     $agent->field( "pin",          $values{'PASSWORD'} );
     $agent->click();
 
-    print STDERR Dumper( $agent->content ) if $self->Debug();
-    
-    if ($agent->content =~ /fieldtableerrorred/ig) {
-        return undef;
-    }
-
-    return 1;
- 
+    print STDERR Dumper( $agent->content ) if Debug();
 }
 
 sub saldo {
     my $self = shift;
     my $data = $self->statement(@_);
-   
-    if ($data) { 
-        print STDERR Dumper($data) if $self->Debug();
-        return $data->{'STATEMENT'}{'SALDO'};
-    } else {
-        return undef;
-    }
+    print STDERR Dumper($data) if Debug();
+    return $data->{'STATEMENT'}{'SALDO'};
 }
 
 sub statement {
@@ -100,10 +92,6 @@ sub statement {
         @_
     );
 
-    # get mainpage
-    my $login_status = $self->login();
-    return undef unless $login_status;
-       
     my $agent = $self->AGENT();
 
 #   If you've problems with your environmet settings activate this and "use encodings"
@@ -124,7 +112,7 @@ sub statement {
     $agent->get( $self->BASE_URL() . "/umsatzdownload.do" );
 
     my $content = $agent->content();
-    print STDERR Dumper($content) if $self->Debug();
+    print STDERR Dumper($content) if Debug();
 
     my $csv_content = $self->_parse_csv($content);
     return $csv_content;
@@ -144,80 +132,50 @@ sub transfer {
         TAN              => "",
         @_
     );
-    
-    # get mainpage
-    my $login_status = $self->login();
-    return undef unless $login_status;
 
     my $agent = $self->AGENT();
     my $url   = $self->BASE_URL();
 
-    $agent->get($url . "ueberweisung_per_heute_neu.do");
+    $agent->get( $url . "ueberweisung_per_heute_neu.do" );
 
     ( $values{'AMOUNT_EURO'}, $values{'AMOUNT_CENT'} ) =
       split( /\.|,/, $values{'AMOUNT'} );
-    
-    $values{'AMOUNT_CENT'} = sprintf("%02d", $values{'AMOUNT_CENT'});
+    $values{'AMOUNT_CENT'} = sprintf( "%02d", $values{'AMOUNT_CENT'} );
+
+    $agent->field( "auftraggeberKontonummer", $values{'SENDER_ACCOUNT'} );
+
+    $agent->field( "empfaengerName", $values{'RECEIVER_NAME'} );
+
+#   $agent->field("empfaengerDatenSpeichern", $values{'RECEIVER_SAVE'}); # waiting for WWW::Mechanize support
+    $agent->field( "empfaengerBankleitzahl", $values{'RECEIVER_BLZ'} );
+    $agent->field( "empfaengerKontonummer",  $values{'RECEIVER_ACCOUNT'} );
+
+    $agent->field( "betragEuro", $values{'AMOUNT_EURO'} );
+    $agent->field( "betragCent", $values{'AMOUNT_CENT'} );
+
+    $agent->field( "verwendungszweck1", $values{'COMMENT_1'} );
+    $agent->field( "verwendungszweck2", $values{'COMMENT_2'} );
 
-    $agent->field("auftraggeberKontonummer", $values{'SENDER_ACCOUNT'});
-    $agent->field("empfaengerName",          $values{'RECEIVER_NAME'});
-    $agent->field("empfaengerBankleitzahl",  $values{'RECEIVER_BLZ'});
-    $agent->field("empfaengerKontonummer",   $values{'RECEIVER_ACCOUNT'});
-    $agent->field("betragEuro",              $values{'AMOUNT_EURO'});
-    $agent->field("betragCent",              $values{'AMOUNT_CENT'});
-    $agent->field("verwendungszweck1",       $values{'COMMENT_1'});
-    $agent->field("verwendungszweck2",       $values{'COMMENT_2'});
     $agent->click("btnNeuSpeichern");
 
-    # get TAN fieldname, index
-    my $tan_field;
-    my $tan_index;
-    my $tan;
-    
-    $agent->content =~ /<label\s*for=["']*(.*?)["']\s*>(?:Tr|TAN)(?:[a-z].*?)\s?(\d+)/gmix;
-   
-    $tan_field = $1;
-    $tan_index = $2;
-   
-    print STDERR "tan_field: $tan_field, tan_index: $tan_index\n" if $self->Debug();
-    
-    if ($tan_field && $tan_index && ref($values{'TAN'}) eq "HASH" && $values{'TAN'}->{$tan_index}) {
-        print STDERR "METHOD: TAN HASH" if $self->Debug();
-        $tan = $values{'TAN'}->{$tan_index};
-    } elsif ($tan_field && $tan_index && ref($values{'TAN'}) eq "ARRAY") {
-        print STDERR "METHOD: Object/Method" if $self->Debug();
-        my $obj    = $values{'TAN'}[0];
-        my $method = $values{'TAN'}[1];
-        eval ( $tan = $obj->$method() );
-        if ($@) {
-            print "ERROR: Could not execute TAN method: $method";
-            return undef;
-        }
-    } elsif ($tan_field && $tan_index && ref($values{'TAN'}) eq "CODE") {
-        print STDERR "METHOD: CALLBACK" if $self->Debug();
-        $tan = &{$values{'TAN'}}($tan_index);
+    # sure it's right ...
+
+    $agent->field( "ubo", $values{'TAN'} );
+    $agent->click("btnBestaetigen");
+
+    # lazy error checking
+
+    if ( $agent->content() =~ m|<span class="error">(.*)30017(.*)</span>| ) {
+        $agent->content() =~ m|<span class="error">(.*?)</span>|;
+        my $error = $1;
+        print "ERROR: $error";
+        return ( 0, 0 );
     }
 
-    print STDERR " TAN: [$tan]\n" if $self->Debug();
-    
-    if ($tan) {
-	    $agent->field($tan_field, $tan_index);
-	    $agent->click("btnBestaetigen");
-
-        # lazy error checking
-        if ( $agent->content() =~ m|<span class="error">(.*)30017(.*)</span>| ) {
-            $agent->content() =~ m|<span class="error">(.*?)</span>|;
-            my $error = $1;
-            print "ERROR: $error";
-            return;
-        } else {
-            my $content = $agent->content();
-            return $agent->content();
-        }
-    } else {
-        print "ERROR: Could not identify requested TAN Index #";
-        return;
+    else {
+        ( 1, $agent->content() );
     }
+
 }
 
 sub logout {
@@ -286,34 +244,36 @@ sub _parse_csv {
 
 1;
 __END__
+# Below is the stub of documentation for your module. You better edit it!
+
 
 =head1 NAME
 
-Finance::Bank::DE::NetBank - Check your NetBank Bank Accounts with Perl
+Finance::Bank::DE::NetBank - Check your SpardaBank Bank Accounts with Perl
 
 =head1 SYNOPSIS
 
-    use Finance::Bank::DE::NetBank;
-    
-    my $account = Finance::Bank::DE::NetBank->new(
-        CUSTOMER_ID => '12345678',
-        ACCOUNT => '12345678',
-        PASSWORD => 'ROUTE66',
-    );
-    
-    if ($account->login()) {
-        print $account->saldo();
-        $account->logout();
-    } 
-    else {
-        print 'login failed. manual interaction needed';
-    }
+ use Finance::Bank::DE::NetBank;
+ my $account = Finance::Bank::DE::NetBank->new(
+						 CUSTOMER_ID => "12345678",
+						 ACCOUNT => "12345678",
+						 PASSWORD => "ROUTE66",
+                                                 BLZ => "70090500",
+                                                 );
+ $account->connect(); 
+ $account->login();
+ print $account->saldo();
+ $account->logout();
 
 =head1 DESCRIPTION
 
 
 This module provides a very limited interface to the webbased online banking
 interface of the German "NetBank e.G." operated by Sparda-Datenverarbeitung e.G..
+It will only work with German NetBank accounts - e.g. the Austrian Sparda Bank 
+Accounts will not work.
+
+It uses OOD and doesn't export anything.
 
 B<WARNING!> This module is neither offical nor is it tested to be 100% save! 
 Because of the nature of web-robots, B<everything may break from one day to
@@ -325,7 +285,8 @@ the source of this module yourself to reassure yourself that I am not
 doing anything untoward with your banking data. This software is useful
 to me, but is provided under B<NO GUARANTEE>, explicit or implied.
 
-You can find tests in the C< t > subdirectory. See L< TESTS > for more details.
+You can find some basic test scripts for manual testing against the demo-banking
+accounts within the directory "test" in the source directry.
 
 =head1 METHODS
 
@@ -335,36 +296,41 @@ This constructor will set the default values and/or user provided values for
 connection and authentication.
 
     my $account = Finance::Bank::DE::NetBank->new (
-        CUSTOMER_ID => 'demo',    
-        PASSWORD => '',      
-        ACCOUNT => '2777770',   
-        @_);
+                  BASE_URL => "https://www.bankingonline.de/sparda-banking/view/",
+                  BLZ => "70090500",        
+                  CUSTOMER_ID => "demo",    
+                  PASSWORD => "",      
+                  ACCOUNT => "2777770",   
+                  AGENT_TYPE => "Internet Explorer 6",
+	      , @_);
 
 If you don't provide any values the module will automatically use the demo account.
 
 CUSTOMER_ID is your "Kundennummer" and ACCOUNT is the "Kontonummer" 
 (if you have only one account you can skip that)
 
+
 =head2 $account->Version()
 
 returns the module version
 
 =head2 $account->Debug($value)
 
-Provide a true  C< $value > get some Data::Dumper outputs on STDERR.
+Set $value to 1 to get some Data::Dumper outputs on STDERR.
 
 =head2 $account->connect()
 
-deprecated. use only $account->login()
+This method will create the user agent and connect to the online banking website.
+Also this (done by WWW::Mechanize) automagically handles the session-id handling.
+
+    $account->connect();
 
 =head2 $account->login(%values)
 
 This method will try to log in with the provided authentication details. If
 nothing is specified the values from the constructor or the defaults will be used.
 
-    $account->login(ACCOUNT => '1234');
-
-Returns C< undef > on error.
+    $account->login(ACCOUNT => "1234");
 
 =head2 $account->saldo(%values)
 
@@ -373,9 +339,7 @@ The method uses the account number if previously set.
 
 You can override/set it:
 
-    $account->saldo(ACCOUNT => '5555555');
-
-Returns C< undef > on error.
+    $account->saldo(ACCOUNT => "5555555");
 
 =head2 $account->statement(%values)
 
@@ -386,36 +350,38 @@ The value of TIMEFRAME can be "1" (last day only), "30" (last 30 days only), "al
 START_DATE and END_DATE only).
 
     $account->statement(
-        TIMEFRAME => 'variabel',
-        START_DATE => '10.04.2005',
-        END_DATE => '02.05.2005',
-    );
-
-Returns C< undef > on error.
+                                 TIMEFRAME => "variabel",
+                                 START_DATE => "10.04.2005",
+                                 END_DATE => "02.05.2005",
+			    );
 
 =head2 $account->transfer()
 
-Returns C< undef > on error.
 
 =head2 $account->logout()
 
-well - every login method should have a logout method
+This method will just log out the website and it only exists to keep the module logic clean ;-)
 
-=head1 TESTS
+=head1 USAGE
 
-Since version 1.04 C<Finance::Bank::DE::NetBank> comes with a testsuite.
-It's located in the subdirectory C< t > of the distribution.
-
-To run the tests against the live NetBank demo account use this:
-
-    perl Makefile.PL --livetest
-    make test TEST_VERBOSE=1
-
-The default behaviour is not to test against the live website:
-
-    perl Makefile.PL
-    make test
+ use Finance::Bank::DE::NetBank;
+ use Data::Dumper;
 
+ my $account = Finance::Bank::DE::NetBank->new(
+                                                 BLZ => "70090500",
+                                                 CUSTOMER_ID => "xxxxxxx",
+                                                 ACCOUNT => "yyyyyyy",
+                                                 PASSWORD => "zzzzzz",
+                                                 );
+ $account->connect();
+ $account->login();
+ print Dumper($account->statement(
+                                 TIMEFRAME => "variabel",
+                                 START_DATE => "10.04.2005",
+                                 END_DATE => "02.05.2005",
+ 				 )
+             );
+ $account->logout();
 
 =head1 BUGS
 
@@ -425,30 +391,44 @@ or email the author.
 
 =head1 HISTORY
 
-see file 'Changes'
+1.01 Wed Dec 14 15:00:00 2005
+        - fixed pod errors
+        - enhanced pod
+1.00 Wed Dec 14 00:43:00 2005
+        - changed URL to barrier free version (works without that new captcha)
+        - replaced buggy html scraping of saldo(). saldo() now uses statement() to retrive the value.
+        
+0.02 Sun May 04 15:45:00 2003
+        - documentation fixes
+
+0.01 Sun May 04 03:00:00 2003
+	- original version;
 
 =head1 THANK YOU
 
-Torsten Mueller (updated URL, saldo() bug reporting)
-
-Sascha Stock (reported bad example in POD)
+ Torsten Mueller (updated URL, saldo() bug reporting)
+ Sascha Stock (reported bad example in POD)
 
 =head1 AUTHOR
 
-Roland Moriz (RMORIZ) <rmoriz@cpan.org>
+ Roland Moriz
+ rmoriz@cpan.org
+ http://www.perl-freelancer.de/
 
-http://www.perl-freelancer.de/ 
-
-http://www.roland-moriz.de/
+Disclaimer stolen from Simon Cozens' Finance::Bank::LloydsTSB without asking for permission %-)
 
 =head1 COPYRIGHT
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
 
+The full text of the license can be found in the
+LICENSE file included with this module.
+
+
 =head1 SEE ALSO
 
-WWW::Mechanize
+Finance::Bank::DE::SpardaBank, WWW::Mechanize, Finance::Bank::LloydsTSB
 
 =cut
 
